import {
  select,
  csv,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  scaleOrdinal,
  schemeSpectral
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  const title = 'High-technology exports vs Taxes on international trade';
  
  const cValue = data => data['IncomeGroup'];
  const country = data => data['Country'];
  
  const color = scaleOrdinal(schemeSpectral[4]);
  

  //console.log(color.domain)
  
  var xValue = data => data['High-technology exports (% of manufactured exports)'];
  const xAxisLabel = 'High-technology exports (% of manufactured exports)';
  
  var yValue = data => data['Taxes on international trade (% of revenue)'];
  const circleRadius = 8;
  const yAxisLabel = 'Taxes on international trade (% of revenue)';
  
  const margin = { top: 50, right: 40, bottom: 60, left: 100 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(10);
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -50)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 55)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  const points = g.selectAll('circle').data(data);
  const pointsEnter = points.enter().append('circle')
    
  pointsEnter
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', circleRadius)
  	  .style('fill', d => color(cValue(d)));
  
  points.merge(points)
      .attr('opacity', d =>
        (!selectedLegend || d === selectedLegend) 
            ? 1 
            : 0.05
      );
  
  
  g.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .text(title);
  
  // Reference: http://bl.ocks.org/weiglemc/6185069
  // draw legend
  
  let selectedLegend;
  
  const onClick = d => {
    selectedLegend = d;
 //   console.log({selectedLegend});
    render();
  };
  
   
  const render = () => {
 /* svg.call(scatterplot, {
    xValue: d => d['High-technology exports (% of manufactured exports)'],
    xAxisLabel: 'High-technology exports (% of manufactured exports)',
    yValue: d['Taxes on international trade (% of revenue)'],
    yAxisLabel: 'Taxes on international trade (% of revenue)',
    circleRadius: 8,
    margin: { top: 50, right: 40, bottom: 60, left: 100 },
    width,
    height,
    data,
    color,
    cValue,
    selectedLegend
  });
 */   
    legend
      .merge(g)
        .attr('opacity', d => 
          (!selectedLegend || d === selectedLegend)
              ? 1
              : 0.2
         )
        .on('click', d => onClick(
          d === selectedLegend
          ? null
          : d
          ));
  };
  
  var legend = svg.selectAll(".legend")
      .data(color.domain())
    .enter().append("g")
      .attr("class", "legend")
      .attr("transform", (d, i) => `translate(-40,${(i + 8.5) * 20})`)
    .merge(g)
      .attr('opacity', d => 
            (!selectedLegend || d === selectedLegend)
              ? 1
              : 0.2
           )
      .on('click', d => onClick(
        d === selectedLegend
          ? null
          : d
      ));

  // draw legend colored rectangles
  legend.append("rect")
      .attr("x", width - 18)
      .attr("y", -110)
      .attr("width", 18)
      .attr("height", 18)
      .style("fill", color);

  // draw legend text
  legend.append("text")
      .attr("x", width - 24)
      .attr("y", -105)
      .attr("dy", ".6em")
      .style("text-anchor", "end")
      .text(function(d) { return d;})
};

  
csv('https://vizhub.com/veagy/datasets/3-tax-vs-high-tech-exp.csv')
  .then(data => {
    data.forEach(d => {
      d['High-technology exports (% of manufactured exports)'] = +d['High-technology exports (% of manufactured exports)'];
      d['Taxes on international trade (% of revenue)'] = +d['Taxes on international trade (% of revenue)'];
    });
    render(data);
  });